@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
+# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
 
 """
 Packet handling
@@ -32,13 +32,13 @@ from paramiko import util
 from paramiko.common import (
     linefeed_byte,
     cr_byte_value,
-    asbytes,
     MSG_NAMES,
     DEBUG,
     xffffffff,
     zero_byte,
+    byte_ord,
 )
-from paramiko.py3compat import u, byte_ord
+from paramiko.util import u
 from paramiko.ssh_exception import SSHException, ProxyCommandFailure
 from paramiko.message import Message
 
@@ -62,7 +62,7 @@ def first_arg(e):
     return arg
 
 
-class Packetizer(object):
+class Packetizer:
     """
     Implementation of the base SSH packet protocol.
     """
@@ -86,6 +86,7 @@ class Packetizer(object):
         self.__need_rekey = False
         self.__init_count = 0
         self.__remainder = bytes()
+        self._initial_kex_done = False
 
         # used for noticing when to re-key:
         self.__sent_bytes = 0
@@ -111,6 +112,14 @@ class Packetizer(object):
         self.__compress_engine_in = None
         self.__sequence_number_out = 0
         self.__sequence_number_in = 0
+        self.__etm_out = False
+        self.__etm_in = False
+
+        # AEAD (eg aes128-gcm/aes256-gcm) cipher use
+        self.__aead_out = False
+        self.__aead_in = False
+        self.__iv_out = None
+        self.__iv_in = None
 
         # lock around outbound writes (packet computation)
         self.__write_lock = threading.RLock()
@@ -128,6 +137,12 @@ class Packetizer(object):
     def closed(self):
         return self.__closed
 
+    def reset_seqno_out(self):
+        self.__sequence_number_out = 0
+
+    def reset_seqno_in(self):
+        self.__sequence_number_in = 0
+
     def set_log(self, log):
         """
         Set the Python log object to use for logging.
@@ -142,9 +157,13 @@ class Packetizer(object):
         mac_size,
         mac_key,
         sdctr=False,
+        etm=False,
+        aead=False,
+        iv_out=None,
     ):
         """
         Switch outbound data cipher.
+        :param etm: Set encrypt-then-mac from OpenSSH
         """
         self.__block_engine_out = block_engine
         self.__sdctr_out = sdctr
@@ -154,6 +173,9 @@ class Packetizer(object):
         self.__mac_key_out = mac_key
         self.__sent_bytes = 0
         self.__sent_packets = 0
+        self.__etm_out = etm
+        self.__aead_out = aead
+        self.__iv_out = iv_out
         # wait until the reset happens in both directions before clearing
         # rekey flag
         self.__init_count |= 1
@@ -162,10 +184,19 @@ class Packetizer(object):
             self.__need_rekey = False
 
     def set_inbound_cipher(
-        self, block_engine, block_size, mac_engine, mac_size, mac_key
+        self,
+        block_engine,
+        block_size,
+        mac_engine,
+        mac_size,
+        mac_key,
+        etm=False,
+        aead=False,
+        iv_in=None,
     ):
         """
         Switch inbound data cipher.
+        :param etm: Set encrypt-then-mac from OpenSSH
         """
         self.__block_engine_in = block_engine
         self.__block_size_in = block_size
@@ -176,6 +207,9 @@ class Packetizer(object):
         self.__received_packets = 0
         self.__received_bytes_overflow = 0
         self.__received_packets_overflow = 0
+        self.__etm_in = etm
+        self.__aead_in = aead
+        self.__iv_in = iv_in
         # wait until the reset happens in both directions before clearing
         # rekey flag
         self.__init_count |= 2
@@ -299,9 +333,6 @@ class Packetizer(object):
                 arg = first_arg(e)
                 if arg == errno.EAGAIN:
                     got_timeout = True
-                elif arg == errno.EINTR:
-                    # syscall interrupted; try again
-                    pass
                 elif self.__closed:
                     raise EOFError()
                 else:
@@ -327,9 +358,6 @@ class Packetizer(object):
                 arg = first_arg(e)
                 if arg == errno.EAGAIN:
                     retry_write = True
-                elif arg == errno.EINTR:
-                    # syscall interrupted; try again
-                    retry_write = True
                 else:
                     n = -1
             except ProxyCommandFailure:
@@ -372,17 +400,28 @@ class Packetizer(object):
             buf = buf[:-1]
         return u(buf)
 
+    def _inc_iv_counter(self, iv):
+        # Per https://www.rfc-editor.org/rfc/rfc5647.html#section-7.1 ,
+        # we increment the last 8 bytes of the 12-byte IV...
+        iv_counter_b = iv[4:]
+        iv_counter = int.from_bytes(iv_counter_b, "big")
+        inc_iv_counter = iv_counter + 1
+        inc_iv_counter_b = inc_iv_counter.to_bytes(8, "big")
+        # ...then re-concatenate it with the static first 4 bytes
+        new_iv = iv[0:4] + inc_iv_counter_b
+        return new_iv
+
     def send_message(self, data):
         """
         Write a block of data using the current cipher, as an SSH block.
         """
         # encrypt this sucka
-        data = asbytes(data)
+        data = data.asbytes()
         cmd = byte_ord(data[0])
         if cmd in MSG_NAMES:
             cmd_name = MSG_NAMES[cmd]
         else:
-            cmd_name = "$%x" % cmd
+            cmd_name = "${:x}".format(cmd)
         orig_len = len(data)
         self.__write_lock.acquire()
         try:
@@ -392,24 +431,40 @@ class Packetizer(object):
             if self.__dump_packets:
                 self._log(
                     DEBUG,
-                    "Write packet <%s>, length %d" % (cmd_name, orig_len),
+                    "Write packet <{}>, length {}".format(cmd_name, orig_len),
                 )
                 self._log(DEBUG, util.format_binary(packet, "OUT: "))
             if self.__block_engine_out is not None:
-                out = self.__block_engine_out.update(packet)
+                if self.__etm_out:
+                    # packet length is not encrypted in EtM
+                    out = packet[0:4] + self.__block_engine_out.update(
+                        packet[4:]
+                    )
+                elif self.__aead_out:
+                    # Packet-length field is used as the 'associated data'
+                    # under AES-GCM, so like EtM, it's not encrypted. See
+                    # https://www.rfc-editor.org/rfc/rfc5647#section-7.3
+                    out = packet[0:4] + self.__block_engine_out.encrypt(
+                        self.__iv_out, packet[4:], packet[0:4]
+                    )
+                    self.__iv_out = self._inc_iv_counter(self.__iv_out)
+                else:
+                    out = self.__block_engine_out.update(packet)
             else:
                 out = packet
-            # + mac
-            if self.__block_engine_out is not None:
-                payload = (
-                    struct.pack(">I", self.__sequence_number_out) + packet
-                )
+            # Append an MAC when needed (eg, not under AES-GCM)
+            if self.__block_engine_out is not None and not self.__aead_out:
+                packed = struct.pack(">I", self.__sequence_number_out)
+                payload = packed + (out if self.__etm_out else packet)
                 out += compute_hmac(
                     self.__mac_key_out, payload, self.__mac_engine_out
                 )[: self.__mac_size_out]
-            self.__sequence_number_out = (
-                self.__sequence_number_out + 1
-            ) & xffffffff
+            next_seq = (self.__sequence_number_out + 1) & xffffffff
+            if next_seq == 0 and not self._initial_kex_done:
+                raise SSHException(
+                    "Sequence number rolled over during initial kex!"
+                )
+            self.__sequence_number_out = next_seq
             self.write_all(out)
 
             self.__sent_bytes += len(out)
@@ -420,10 +475,9 @@ class Packetizer(object):
             )
             if sent_too_much and not self.__need_rekey:
                 # only ask once for rekeying
+                msg = "Rekeying (hit {} packets, {} bytes sent)"
                 self._log(
-                    DEBUG,
-                    "Rekeying (hit %d packets, %d bytes sent)"
-                    % (self.__sent_packets, self.__sent_bytes),
+                    DEBUG, msg.format(self.__sent_packets, self.__sent_bytes)
                 )
                 self.__received_bytes_overflow = 0
                 self.__received_packets_overflow = 0
@@ -440,26 +494,68 @@ class Packetizer(object):
         :raises: `.NeedRekeyException` -- if the transport should rekey
         """
         header = self.read_all(self.__block_size_in, check_rekey=True)
-        if self.__block_engine_in is not None:
+        if self.__etm_in:
+            packet_size = struct.unpack(">I", header[:4])[0]
+            remaining = packet_size - self.__block_size_in + 4
+            packet = header[4:] + self.read_all(remaining, check_rekey=False)
+            mac = self.read_all(self.__mac_size_in, check_rekey=False)
+            mac_payload = (
+                struct.pack(">II", self.__sequence_number_in, packet_size)
+                + packet
+            )
+            my_mac = compute_hmac(
+                self.__mac_key_in, mac_payload, self.__mac_engine_in
+            )[: self.__mac_size_in]
+            if not util.constant_time_bytes_eq(my_mac, mac):
+                raise SSHException("Mismatched MAC")
+            header = packet
+
+        if self.__aead_in:
+            # Grab unencrypted (considered 'additional data' under GCM) packet
+            # length.
+            packet_size = struct.unpack(">I", header[:4])[0]
+            aad = header[:4]
+            remaining = (
+                packet_size - self.__block_size_in + 4 + self.__mac_size_in
+            )
+            packet = header[4:] + self.read_all(remaining, check_rekey=False)
+            header = self.__block_engine_in.decrypt(self.__iv_in, packet, aad)
+
+            self.__iv_in = self._inc_iv_counter(self.__iv_in)
+
+        if self.__block_engine_in is not None and not self.__aead_in:
             header = self.__block_engine_in.update(header)
         if self.__dump_packets:
             self._log(DEBUG, util.format_binary(header, "IN: "))
-        packet_size = struct.unpack(">I", header[:4])[0]
-        # leftover contains decrypted bytes from the first block (after the
-        # length field)
-        leftover = header[4:]
-        if (packet_size - len(leftover)) % self.__block_size_in != 0:
-            raise SSHException("Invalid packet blocking")
-        buf = self.read_all(packet_size + self.__mac_size_in - len(leftover))
-        packet = buf[: packet_size - len(leftover)]
-        post_packet = buf[packet_size - len(leftover) :]
-        if self.__block_engine_in is not None:
-            packet = self.__block_engine_in.update(packet)
+
+        # When ETM or AEAD (GCM) are in use, we've already read the packet size
+        # & decrypted everything, so just set the packet back to the header we
+        # obtained.
+        if self.__etm_in or self.__aead_in:
+            packet = header
+        # Otherwise, use the older non-ETM logic
+        else:
+            packet_size = struct.unpack(">I", header[:4])[0]
+
+            # leftover contains decrypted bytes from the first block (after the
+            # length field)
+            leftover = header[4:]
+            if (packet_size - len(leftover)) % self.__block_size_in != 0:
+                raise SSHException("Invalid packet blocking")
+            buf = self.read_all(
+                packet_size + self.__mac_size_in - len(leftover)
+            )
+            packet = buf[: packet_size - len(leftover)]
+            post_packet = buf[packet_size - len(leftover) :]
+
+            if self.__block_engine_in is not None:
+                packet = self.__block_engine_in.update(packet)
+            packet = leftover + packet
+
         if self.__dump_packets:
             self._log(DEBUG, util.format_binary(packet, "IN: "))
-        packet = leftover + packet
 
-        if self.__mac_size_in > 0:
+        if self.__mac_size_in > 0 and not self.__etm_in and not self.__aead_in:
             mac = post_packet[: self.__mac_size_in]
             mac_payload = (
                 struct.pack(">II", self.__sequence_number_in, packet_size)
@@ -476,7 +572,9 @@ class Packetizer(object):
         if self.__dump_packets:
             self._log(
                 DEBUG,
-                "Got payload (%d bytes, %d padding)" % (packet_size, padding),
+                "Got payload ({} bytes, {} padding)".format(
+                    packet_size, padding
+                ),
             )
 
         if self.__compress_engine_in is not None:
@@ -484,7 +582,12 @@ class Packetizer(object):
 
         msg = Message(payload[1:])
         msg.seqno = self.__sequence_number_in
-        self.__sequence_number_in = (self.__sequence_number_in + 1) & xffffffff
+        next_seq = (self.__sequence_number_in + 1) & xffffffff
+        if next_seq == 0 and not self._initial_kex_done:
+            raise SSHException(
+                "Sequence number rolled over during initial kex!"
+            )
+        self.__sequence_number_in = next_seq
 
         # check for rekey
         raw_packet_size = packet_size + self.__mac_size_in + 4
@@ -508,10 +611,10 @@ class Packetizer(object):
             self.__received_bytes >= self.REKEY_BYTES
         ):
             # only ask once for rekeying
+            err = "Rekeying (hit {} packets, {} bytes received)"
             self._log(
                 DEBUG,
-                "Rekeying (hit %d packets, %d bytes received)"
-                % (self.__received_packets, self.__received_bytes),
+                err.format(self.__received_packets, self.__received_bytes),
             )
             self.__received_bytes_overflow = 0
             self.__received_packets_overflow = 0
@@ -521,10 +624,11 @@ class Packetizer(object):
         if cmd in MSG_NAMES:
             cmd_name = MSG_NAMES[cmd]
         else:
-            cmd_name = "$%x" % cmd
+            cmd_name = "${:x}".format(cmd)
         if self.__dump_packets:
             self._log(
-                DEBUG, "Read packet <%s>, length %d" % (cmd_name, len(payload))
+                DEBUG,
+                "Read packet <{}>, length {}".format(cmd_name, len(payload)),
             )
         return cmd, msg
 
@@ -562,11 +666,6 @@ class Packetizer(object):
                 break
             except socket.timeout:
                 pass
-            except EnvironmentError as e:
-                if first_arg(e) == errno.EINTR:
-                    pass
-                else:
-                    raise
             if self.__closed:
                 raise EOFError()
             now = time.time()
@@ -577,7 +676,10 @@ class Packetizer(object):
     def _build_packet(self, payload):
         # pad up at least 4 bytes, to nearest block-size (usually 8)
         bsize = self.__block_size_out
-        padding = 3 + bsize - ((len(payload) + 8) % bsize)
+        # do not include payload length in computations for padding in EtM mode
+        # (payload length won't be encrypted)
+        addlen = 4 if self.__etm_out or self.__aead_out else 8
+        padding = 3 + bsize - ((len(payload) + addlen) % bsize)
         packet = struct.pack(">IB", len(payload) + padding + 1, padding)
         packet += payload
         if self.__sdctr_out or self.__block_engine_out is None:
