@@ -24,6 +24,7 @@ from binascii import hexlify
 import os
 from hashlib import sha1
 import unittest
+from importlib import metadata
 
 import paramiko
 import paramiko.util
@@ -43,7 +44,7 @@ BGQ3GQ/Fc7SX6gkpXkwcZryoi4kNFhHu5LvHcZPdxXV1D+uTMfGS1eyd2Yz/DoNWXNAl8TI0cAsW\
 class UtilTest(unittest.TestCase):
     def test_imports(self):
         """
-        verify that all the classes can be imported from paramiko.
+        Verify that all the classes can be imported from paramiko.
         """
         for name in (
             "Agent",
@@ -61,7 +62,8 @@ class UtilTest(unittest.TestCase):
             "ChannelException",
             "ConfigParseError",
             "CouldNotCanonicalize",
-            "DSSKey",
+            "ECDSAKey",
+            "Ed25519Key",
             "HostKeys",
             "InMemoryPrivateKey",
             "Message",
@@ -95,6 +97,9 @@ class UtilTest(unittest.TestCase):
         ):
             assert name in dir(paramiko)
 
+    def test_version_available_in_main_module(self):
+        assert paramiko.__version__ == metadata.version("paramiko")
+
     def test_generate_key_bytes(self):
         key_bytes = paramiko.util.generate_key_bytes(
             sha1, b"ABCDEFGH", "This is my secret passphrase.", 64
